package me.totalfreedom.discord.listener;

import com.google.common.base.Strings;
import discord4j.core.event.domain.message.MessageCreateEvent;
import discord4j.core.object.entity.Attachment;
import discord4j.core.object.entity.Guild;
import discord4j.core.object.entity.Member;
import discord4j.core.object.entity.Message;
import me.totalfreedom.discord.Bot;
import me.totalfreedom.discord.TFD4J;
import me.totalfreedom.discord.util.SnowflakeEntry;
import me.totalfreedom.totalfreedommod.TotalFreedomMod;
import me.totalfreedom.totalfreedommod.admin.Admin;
import me.totalfreedom.totalfreedommod.rank.Displayable;
import me.totalfreedom.totalfreedommod.rank.Rank;
import me.totalfreedom.totalfreedommod.rank.Title;
import me.totalfreedom.totalfreedommod.util.FLog;
import me.totalfreedom.totalfreedommod.util.FUtil;
import net.md_5.bungee.api.chat.ClickEvent;
import net.md_5.bungee.api.chat.ComponentBuilder;
import net.md_5.bungee.api.chat.TextComponent;
import org.bukkit.Bukkit;
import org.bukkit.ChatColor;

public class AdminChatListener
{
    private final Bot bot;
    private final TFD4J tfd4j;

    public AdminChatListener(TFD4J tfd4j)
    {
        this.tfd4j = tfd4j;
        this.bot = tfd4j.getBot();
    }

    public static net.md_5.bungee.api.ChatColor getColor(Displayable display)
    {
        return display.getColor();
    }

    public void adminChatBound()
    {
        tfd4j.getBot()
                .getClient()
                .getEventDispatcher()
                .on(MessageCreateEvent.class)
                .filter(m -> m.getMessage()
                        .getChannel()
                        .blockOptional()
                        .orElseThrow()
                        .getId()
                        .equals(SnowflakeEntry.adminChatChannelID))
                .filter(m -> !m.getMessage()
                        .getAuthor()
                        .orElseThrow()
                        .getId()
                        .equals(tfd4j.getBot().getClient().getSelfId()))
                .subscribe(this::createMessageSpec);
    }

    public void createMessageSpec(MessageCreateEvent m)
    {
        Member member = m.getMember().orElseThrow(IllegalAccessError::new);
        String tag = tfd4j.getMinecraftListener().getDisplay(member);
        Message msg = m.getMessage();
        String mediamessage = ChatColor.YELLOW + "[Media]";

        StringBuilder logmessage = new StringBuilder(ChatColor.DARK_GRAY + "[" + ChatColor.DARK_AQUA + "Discord" + ChatColor.DARK_GRAY + "] " + ChatColor.RESET);
        String lm = ChatColor.DARK_RED + member.getDisplayName() + " "
                + ChatColor.DARK_GRAY + tag + ChatColor.DARK_GRAY
                + ChatColor.WHITE + ": " + ChatColor.GOLD + FUtil.colorize(msg.getContent());
        logmessage.append(lm);

        if (!msg.getAttachments().isEmpty())
        {

            logmessage.append(mediamessage); // Actually for logging...

        }
        FLog.info(logmessage.toString());

        Bukkit.getOnlinePlayers().stream().filter(player -> TotalFreedomMod.getPlugin().al.isAdmin(player)).forEach(player ->
        {
            StringBuilder message = new StringBuilder(ChatColor.DARK_GRAY + "[" + ChatColor.DARK_AQUA + "Discord" + ChatColor.DARK_GRAY + "] " + ChatColor.RESET);

            ComponentBuilder builder = new ComponentBuilder(message.toString());

            Admin admin = TotalFreedomMod.getPlugin().al.getAdmin(player);
            String format = admin.getAcFormat();
            if (!Strings.isNullOrEmpty(format))
            {
                Displayable display = getDisplay(member);
                net.md_5.bungee.api.ChatColor color = getColor(display);
                String m1 = format.replace("%name%", member.getDisplayName())
                        .replace("%rank%", display.getAbbr())
                        .replace("%rankcolor%", color.toString())
                        .replace("%msg%", FUtil.colorize(msg.getContent()));
                builder.append(FUtil.colorize(m1));

            } else
            {
                String m1 = ChatColor.DARK_RED + member.getDisplayName() + " "
                        + ChatColor.DARK_GRAY + tag + ChatColor.DARK_GRAY
                        + ChatColor.WHITE + ": " + ChatColor.GOLD + FUtil.colorize(msg.getContent());
                builder.append(m1);
            }

            if (!msg.getAttachments().isEmpty())
            {
                for (Attachment attachment : msg.getAttachments())
                {
                    TextComponent text = new TextComponent(mediamessage);
                    text.setClickEvent(new ClickEvent(ClickEvent.Action.OPEN_URL, attachment.getUrl()));
                    if (!msg.getContent().isEmpty())
                    {
                        builder.append(" ");
                    }
                    builder.append(text);
                }
            }
            player.spigot().sendMessage(builder.create());
        });
    }

    public Displayable getDisplay(Member member)
    {
        Guild server = tfd4j.getBot().getGuildById().block();
        // Server Owner
        assert server != null;
        return member.getRoles().map(role ->
        {
            if (role.getId().equals(SnowflakeEntry.ownerRoleID))
            {
                return Title.OWNER;
            } else if (role.getId().equals(SnowflakeEntry.developerRoleID))
            {
                return Title.DEVELOPER;
            } else if (role.getId().equals(SnowflakeEntry.executiveRoleID))
            {
                return Title.EXECUTIVE;
            } else if (role.getId().equals(SnowflakeEntry.assistantRoleID))
            {
                return Title.ASSTEXEC;
            } else if (role.getId().equals(SnowflakeEntry.seniorRoleID))
            {
                return Rank.SENIOR_ADMIN;
            } else if (role.getId().equals(SnowflakeEntry.adminRoleID))
            {
                return Rank.ADMIN;
            } else if (role.getId().equals(SnowflakeEntry.builderRoleID))
            {
                return Title.MASTER_BUILDER;
            } else
            {
                return null;
            }
        }).blockFirst();
    }
}
